%% Least Squares.

% How can we determine the relationship between the admission exam and a
% course grades for any student in a university? We have the following
% dataset obtained from 22 students:
D = [...
%- Exam grade ------ Course grade -%
    50                  53;...
    35                  41;...
    35                  61;...
    40                  56;...
    55                  68;...
    65                  36;...
    35                  11;...
    60                  70;...
    90                  79;...
    35                  59;...
    90                  54;...
    80                  91;...
    60                  48;...
    60                  71;...
    60                  71;...
    40                  47;...
    55                  53;...
    50                  68;...
    65                  57;...
    50                  79];
%---------------------------------%

x = D(:,1);             % Independent variable.
y = D(:,2);             % Dependent variable.

figure;
plot( x, y, 'bo', 'MarkerFaceColor', 'g', 'MarkerSize', 7 );
xlabel( 'x' );
ylabel( 'y' );
title( 'Mathematical relationship between admission exam grade and a course grade' );
xlim( [min(x)-10 max(x)+10] );
ylim( [min(y)-10 max(y)+10] );
axis equal;
hold on;

%% Solving the least squares problem.
n = max( size(D) );
A = [ x, ones(n,1) ];

% Ac = y <---- where c is a vector of two coefficients: a and b, for a line
% with equation y = ax + b
c = pinv(A) * y;

%% Showing the best fit for the input dataset.

xLine = [0; 120];
ALine = [xLine ones(2,1)];
yLine = ALine * c;
plot( xLine, yLine, 'm-' );
text( 10, 80, sprintf('y = %0.3fx + %0.3f', c(1), c(2) ), 'Color', 'm', 'FontSize', 12 );

%% Computing the error.

yNew = A * c;           % Approximated y.
for I = 1:n
    plot( [x(I) x(I)], [yNew(I) y(I)], '--', 'Color', [0.6,0.6,0.6] );
end;
plot( x, yNew, 'b.', 'MarkerSize', 10 );

error = norm( yNew - y )^2;
text( 80, 10, sprintf( 'Error = %.3f', error ), 'Color', 'k', 'FontSize', 12 );
hold off;
