%% Transformations and SVD.
T = [1.5 2 1;...        % Transformation matrix.
     2.1 1 0.5];
 
% Generate a unit sphere in 3D space.
X = [];
Y = [];
Z = [];
r = 1;
phi = linspace( 0, pi, 20 )';
for theta = linspace( 0, 2*pi, 40 )
    x = r * cos(theta) * sin(phi);
    y = r * sin(theta) * sin(phi);
    z = r * cos(phi);
    X = [X; x];
    Y = [Y; y];
    Z = [Z; z];
end;

% Plot the unit sphere.
figure;
plot3( X, Y, Z, 'm.', 'MarkerSize', 10 );
axis equal;
xlabel( 'x' );
ylabel( 'y' );
zlabel( 'z' );
xlim( [-3, 3] );
ylim( [-3, 3] );
zlim( [-3, 3] );
grid on;
hold on;

% Transform the unit sphere.
% Notice that we will flatten it because T is a 2x3 matrix.
D = [X'; Y'; Z'];
TD = T * D;
TX = TD(1,:)';
TY = TD(2,:)';
TZ = zeros( size( TX ) );
plot3( TX, TY, TZ, '.', 'MarkerSize', 10, 'Color', [0.9,0.8,0.9] );
legend( 'Unit sphere', 'Transformed unit sphere' );
title( 'Transforming the unit sphere' );
hold off;

%% SVD of the transformation matrix T.
[U, S, V] = svd( T );

figure;
plot( TX, TY, '.', 'MarkerSize', 10, 'Color', [0.9,0.8,0.9] );
axis equal;
xlim( [-4, 4] );
ylim( [-4, 4] );
hold on;
e1 = U(:,1) * S(1,1);       % Axis with largest eigenvalue.
e2 = U(:,2) * S(2,2);       % Second and last axis with nonzero eigenvalue.
quiver( 0, 0, e1(1), e1(2), 'b', 'LineWidth', 2 );
text( e1(1)*1.1, e1(2)*1.2, 'u_1 * \sigma_1', 'FontSize', 12, 'Color', 'b' );
quiver( 0, 0, e2(1), e2(2), 'r', 'LineWidth', 2 );
text( e2(1)-1, e2(2)*1.3, 'u_2 * \sigma_2', 'FontSize', 12, 'Color', 'r' );
legend( 'Transformed unit sphere',...
    '1st left singular vector\times \sigma_1',...
    '2nd left singular vector\times \sigma_2' );
title( 'SVD' );
hold off;
