%% Manipulating images with SVD.

% Reading an image.
taeyeon = imread( 'taeyeon.jpg' );      % uint8 values in the range of [0,255].
taeyeon = double( taeyeon ) / 255;      % Normalize values to [0,1].

% Making it grayscale (use im2gray, alternatively).
R = taeyeon( :, :, 1 );     % Red channel.
G = taeyeon( :, :, 2 );     % Green channel.
B = taeyeon( :, :, 3 );     % Blue channel.
T = (R + G + B)/3;

% Display R, G, and B channels.
figure;
subplot( 1, 4, 1 );
imshow( taeyeon );
title( 'Taeyeon, RGB full image' );
subplot( 1, 4, 2 );
img = zeros( size( taeyeon ) );         % Notice how we create an RGB, 3D matrix.
img(:,:,1) = R;
imshow( img );
title( 'Taeyeon, red channel' );
subplot( 1, 4, 3 );
img = zeros( size( taeyeon ) );
img(:,:,2) = G;
imshow( img );
title( 'Taeyeon, green channel' );
subplot( 1, 4, 4 );
img = zeros( size( taeyeon ) );
img(:,:,3) = R;
imshow( img );
title( 'Taeyeon, blue channel' );

% Display RGB and gray-scale image.
figure;
imshow( T );
title( 'Gray-scale Taeyeon' );

%% Understanding SVD with Taeyeon!

[U, S, V] = svd( T );       % U is nxn, S is nxp, and V is pxp.

figure;
bar( diag(S), 'FaceColor', [0,0.5,0.5] );
xlabel( '\sigma_i index' );
ylabel( '\sigma_i value' );
title( 'Eigenvalues \sigma_i of Taeyeon' );

%% Using SVD to compress Taeyeon.

k = 10;
Uk = U(:, 1:k);             % Throw away left singular vectors beyond the k^th.
Vk = V(:, 1:k);             % Throw away right singular vectors beyond the k^th.
Sk = S(1:k, 1:k);           % We only need k singular values.

Tk = Uk * Sk * Vk';         % k^th approximation of Taeyeon.
figure;
imshow( Tk );
title( sprintf( 'Approximating Taeyeon with only k = %d singular values', k ) );

%% The (left) pseudoinverse.

Si = S';                    % We just need the pseudoinverse of S.
indexes = find( Si > 0 );
Si( indexes ) = 1./Si(indexes);
TP = V * Si * U';
minTP = min( min( TP, [], 1 ) );    % Normalize values to [0,1]
maxTP = max( max( TP, [], 1 ) );
TPImg = (TP - minTP) / (maxTP - minTP);

figure;
imshow( TPImg );
title( 'Inverse of Taeyeon image -- we lost her!' );









